#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Tue Apr 10 09:35:45 2018
Optimisation of the cost function using the simulated annealing solver.

@author: Jerome Stenger
"""

import os
import sys
import time 
import copy
sys.path.append(os.path.join(os.path.dirname(__file__), '..'))

from signal import signal, SIGPIPE, SIG_DFL
signal(SIGPIPE,SIG_DFL) 

import matplotlib.pyplot as plt

from Solver_Simulated_Annealing import *
from Model import my_model, Cost_Function

#%%
# =============================================================================
# =============================================================================
############################ CALCULATION RUN ##################################
# =============================================================================
# =============================================================================

# mandatory parameters
threshold_max = 5
threshold_min = 3
step= 0.1 

nb_iter = 5 # Number of time Simulated_Annealing is run for each threshold
Xinit=[140,5000] # initial guess, random between bounds should be fine 
name='Simulated Annealing' # Name of the plot

# non mandatory parameters (lower bounds, upper bounds, means, initial temp, schedule, maxiter, niter)
# schedule sould be one of the following : 'Exp', 'Chauchy, 'Fast' or 'Boltzmann'
kwargs={\
'lower':[160],              \
'upper':[3580],             \
'mean':[1870],              \
'schedule':'Boltzmann',     \
'T0':0.01                   \
}


#rewritte the function to be implemented in the solver
def costSA(x0):
	global threshold, kwargs
	x=[[x0[2*t],x0[2*t+1]] for t in range(int(len(x0)/2))] #Concatenation of the vector per variable
	return Cost_Function(x, threshold=threshold, **kwargs)


# live printing the result
sol=(Xinit,1)
threshold = threshold_max
xmin=[]
xmax=[]
for i in range(len(kwargs.get('lower', None))):
	xmin.append(kwargs.get('lower', None)[i]);xmin.append(kwargs.get('mean', None)[i])
	xmax.append(kwargs.get('mean', None)[i]);xmax.append(kwargs.get('upper', None)[i])
params = copy.deepcopy(kwargs)
params['lower']=xmin
params['upper']=xmax
x=[]
y=[]
plt.rc('text', usetex=True)
plt.rc('font', family='serif')
plt.rc('xtick', labelsize=20) 
plt.rc('ytick', labelsize=20) 
plt.ylabel(r'$\inf_{\mu\in\mathcal{A}} \; F_{\mu}(h)$',fontsize=24)
plt.xlabel('h',fontsize=24)
plt.title(name,fontsize=24)
plt.show()
axes = plt.gca()
axes.set_xlim(threshold_min,threshold_max) # x-axis bounds
axes.set_ylim(0,1) # y-axis bounds
line, = axes.plot(x, y, 'r-')
start = time.time()
while threshold>=threshold_min:
	for k in range(nb_iter):
		
		ret=Simulated_Annealing(costSA,x0=Xinit, **params)
		
		if ret[1]<sol[1]:
			sol=(ret[0],ret[1])
	x.append(threshold)
	y.append(sol[1])
	line.set_xdata(x) # add new points to the figure
	line.set_ydata(y) 
	plt.draw() #update the figure
	plt.pause(1e-17)
	threshold+=(-step)

plt.show() # keep the figure open at the end
end = time.time()
print('time elapsed :',end - start)
time.sleep(0.5)

#%%
# =============================================================================
# =============================================================================
################### RESULT VISUALISATION AND EXPORTATION ######################
# =============================================================================
# =============================================================================
plt.close('all')


#Print and saved fig to be printed with latex 
plt.figure(figsize=(12,7))
plt.rc('text', usetex=True)
plt.rc('font', family='serif')
plt.rc('xtick', labelsize=20) 
plt.rc('ytick', labelsize=20) 
plt.ylabel(r'$\inf_{\mu\in\mathcal{A}} \; F_{\mu}(h)$',fontsize=24)
plt.xlabel('h',fontsize=24)
plt.title(name,fontsize=24)

#plot the curve
plt.plot(x,y,marker='.')

# pgf to be used with latex, png for direct use
plt.savefig('./saved/'+name+'.pgf')
plt.savefig('./saved/'+name+'.png',dpi=300)


# export data as csv
import pandas as pd
d={'1-Threshold':x,'2-Probability of failure':y}
my_df = pd.DataFrame(data=d)
my_df.to_csv('./saved/'+name+'.csv', index=False, header=False,sep=' ')
print(my_df)
